#!/usr/bin/env python
# coding: utf-8

# In[28]:


### import everything
import tkinter as tk 
import numpy as np
import glob
import matplotlib.pyplot as plt
import pathlib
import os

from scipy.integrate import solve_ivp
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
# Implement the default Matplotlib key bindings.
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from tkinter import *
from tkinter import ttk
from pk_two_comp import pk_two_comp as pk2c

class StartPage():

    def __init__(self, root):
        self.root = root
        self.root.title('PK Optimizer Tool')
        
        #Calculating the center
        w = self.root.winfo_reqwidth()
        h = self.root.winfo_reqheight()
        ws = self.root.winfo_screenwidth()
        hs = self.root.winfo_screenheight()
        x = (ws/2) - (w/2)
        y = (hs/2) - (h/2)
        self.root.geometry('+%d+%d' % (x, y))
        
        #Making buttons 
        self.label = Label(self.root,
                           text="Please choose what application you'd like to explore: ",
                           width=30, wraplength=200).grid(column=1, row=0, sticky='nwes')
        self.compare_btn = Button(self.root, text="Compare!", command=self.compare)
        self.compare_btn.grid(column=1, row=1, sticky='nwes')
        
        self.optimize_btn = Button(self.root, text="Optimize!", command=self.optimize)
        self.optimize_btn.grid(column=1, row=2, sticky='nwes')
        
        self.visualize_btn = Button(self.root, text="Visualize!", command=self.visualize)
        self.visualize_btn.grid(column=1, row=3, sticky='nwes')
        
        self.quitMainWindow = Button(self.root, text="Exit Application", command=self.on_cancel)
        self.quitMainWindow.grid(column=1, row=4, sticky='nwes')

    def compare(self):
        my_gui = pk_compare(self) 
        
    def optimize(self):
        my_gui = pk_optimize(self)
        
    def visualize(self):
        my_gui = pk_visualize(self)
        
    def on_cancel(self):
        self.root.destroy()  

class pk_optimize(Toplevel):
    
    def __init__(self, parent):#, master_root):
        super().__init__(name='pk Optimizer tool')
        self.parent = parent
        
        #Making mainfraim to center the window to the screen
        mainframe = ttk.Frame(self)
        #mainframe.grid(column=0, row=0, sticky=(N, W, E, S))
        mainframe.columnconfigure(0, weight=1)
        mainframe.rowconfigure(0, weight=1)
        
        #Calculating the center
        w = 1100
        h = 600
        ws = self.winfo_screenwidth()
        hs = self.winfo_screenheight()
        x = (ws/2) - (w/2)
        y = (hs/2) - (h/2)
        self.geometry('+%d+%d' % (x, y))
        
        self.pk = pk2c()
        self.geometry('1100x600')
        self.frame1 = Frame(self)
        self.frame2 = Frame(self, padx=10)
        self.frame3 = Frame(self)
        self.frame4 = Frame(self)
        self.frame5 = Frame(self)
        self.frame1.grid(column=0, row=1)
        self.frame2.grid(column=0, row=0)
        self.frame3.grid(column=2, row=1)
        self.frame4.grid(column=2, row=0)
        self.frame5.grid(column=1, row=0)
        
        if os.path.basename(os.path.normpath(pathlib.Path().absolute())) != 'Data':
            os.chdir(pathlib.Path('Data').absolute())

        #Get lists of files
        self.filenames = glob.glob('*.{}'.format('csv'))
        # Combobox creation 
        self.chosen_file = tk.StringVar() 
        self.file_combobox = ttk.Combobox(self.frame5, width = 20, textvariable = self.chosen_file) 

        # Adding combobox drop down list 
        self.file_combobox['values'] = self.filenames
        self.file_combobox.grid(column = 0, row = 1) 
        self.file_combobox.current(0)
        
        #Creating blank canvas to plot input function on
        self.fig1 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        self.canvas1 = FigureCanvasTkAgg(self.fig1, master=self.frame2)
        self.canvas1.get_tk_widget().grid(row=0, column=0)
        self.canvas1.draw()
        
        #Creating blank canvas to plot output function on
        self.fig2 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        self.canvas2 = FigureCanvasTkAgg(self.fig2, master=self.frame4)
        self.canvas2.get_tk_widget().grid(row=0, column=0)
        self.canvas2.draw()
        
        #Creating a bunch of labels and input boxes for initial guesses for parameters
        self.ymax_lbl = Label(self.frame1, text="ymax guess: ")
        self.ymax_lbl.grid(column=0, row=0)
        self.tmax_lbl = Label(self.frame1, text="tmax guess: ")
        self.tmax_lbl.grid(column=0, row=1)
        self.alpha_lbl = Label(self.frame1, text="alpha guess: ")
        self.alpha_lbl.grid(column=0, row=2)
        self.delay_lbl = Label(self.frame1, text="delay guess: ")
        self.delay_lbl.grid(column=0, row=3)

        self.ymax_input = Entry(self.frame1,width=50)
        self.ymax_input.grid(column=1, row=0)
        self.tmax_input = Entry(self.frame1,width=50)
        self.tmax_input.grid(column=1, row=1)
        self.alpha_input = Entry(self.frame1,width=50)
        self.alpha_input.grid(column=1, row=2)
        self.delay_input = Entry(self.frame1,width=50)
        self.delay_input.grid(column=1, row=3)

        self.ymax_opt_lbl = Label(self.frame1)
        self.ymax_opt_lbl.grid(column=2, row=0)
        self.tmax_opt_lbl = Label(self.frame1)
        self.tmax_opt_lbl.grid(column=2, row=1)
        self.alpha_opt_lbl = Label(self.frame1)
        self.alpha_opt_lbl.grid(column=2, row =2)
        self.delay_opt_lbl = Label(self.frame1)
        self.delay_opt_lbl.grid(column=2, row=3)
        
        self.flow_lbl = Label(self.frame3, text="flow guess: ")
        self.flow_lbl.grid(column=0, row=0)
        self.visf_lbl = Label(self.frame3, text="visf guess: ")
        self.visf_lbl.grid(column=0, row=1)
        self.baseline_lbl = Label(self.frame3, text="baseline guess: ")
        self.baseline_lbl.grid(column=0, row=2)

        self.flow_input = Entry(self.frame3,width=50)
        self.flow_input.grid(column=1, row=0)
        self.visf_input = Entry(self.frame3,width=50)
        self.visf_input.grid(column=1, row=1)
        self.baseline_input = Entry(self.frame3,width=50)
        self.baseline_input.grid(column=1, row=2)

        self.flow_opt_lbl = Label(self.frame3)
        self.flow_opt_lbl.grid(column=2, row=0)
        self.visf_opt_lbl = Label(self.frame3)
        self.visf_opt_lbl.grid(column=2, row=1)
        self.baseline_opt_lbl = Label(self.frame3)
        self.baseline_opt_lbl.grid(column=2, row =2)
        
        #Creating buttons to load data and perform optimization
        self.load_btn_text = tk.StringVar()
        self.load_btn = Button(master=self.frame5, textvariable=self.load_btn_text, command=self.load_data)
        self.load_btn_text.set("Load")
        self.load_btn.grid(column=0, row=2)

        self.opt_btn_text = tk.StringVar()
        self.opt_btn = Button(master=self.frame5, textvariable=self.opt_btn_text, command=self.perform_opt)
        self.opt_btn_text.set("Perform Optimization")
        self.opt_btn.grid(column=0, row=3)
        
        #Creating button to go back to main page
        self.back_btn = Button(master=self.frame5, text="Go Back", command=self.go_back)
        self.back_btn.grid(column=0, row=4)

# #-------------------------------------------------------------
# #-------------------------------------------------------------
# #-------------------------------------------------------------

    def load_data(self):
        
        file = self.file_combobox.get()
        self.pk.Get_data(file)

        self.fig1.add_subplot(111).plot(self.pk.time, self.pk.aorta, 'bo')
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.fig1.canvas.draw()

        self.fig2.add_subplot(111).plot(self.pk.time, self.pk.myo, 'ro')
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.canvas2.draw_idle()

    def perform_opt(self):

        #Performs function fitting, outputs optimized parameters in opt
        opt = self.pk.inputFuncFit([self.ymax_input.get(), self.tmax_input.get(),
                                         self.alpha_input.get(), self.delay_input.get()])

        #Shows opt in new labels
        self.ymax_opt_lbl.configure(text=opt[0])
        self.tmax_opt_lbl.configure(text=opt[1])
        self.alpha_opt_lbl.configure(text=opt[2])
        self.delay_opt_lbl.configure(text=opt[3])

        #Plots fitted gamma_var curve
        self.fig1.add_subplot(111).plot(np.arange(0, 25, 0.01),
                                        self.pk.gamma_var(np.arange(0, 25, 0.01), opt[0], opt[1], opt[2], opt[3]), 'k-')
        self.canvas1.draw_idle()

        #Performs function fitting, outputs optimized parameters in opt
        opt = self.pk.outputFuncFit([self.flow_input.get(),
                                     self.visf_input.get(), self.baseline_input.get()])

        #Shows opt in new labels
        self.flow_opt_lbl.configure(text=opt[0])
        self.visf_opt_lbl.configure(text=opt[1])
        self.baseline_opt_lbl.configure(text=opt[2])

        #Plots fitted gamma_var curve
        deriv_sol = solve_ivp(self.pk.derivs, [0, 30], [0, 0], t_eval=self.pk.time)
        fit_myo = deriv_sol.y[0] + deriv_sol.y[1]
        self.fig2.add_subplot(111).plot(self.pk.time, fit_myo + opt[2], 'm-')
        self.canvas2.draw_idle()
        
    def go_back(self):
        self.destroy()

        
class pk_compare(Toplevel):
    
    def __init__(self, parent):#, master_root):
        super().__init__(name='pk Optimizer tool')
        self.parent = parent
        self.legend = []
        
        #Making mainfraim to center the window to the screen
        mainframe = ttk.Frame(self)
        #mainframe.grid(column=0, row=0, sticky=(N, W, E, S))
        mainframe.columnconfigure(0, weight=1)
        mainframe.rowconfigure(0, weight=1)
        
        #Calculating the center
        w = 1100
        h = 500
        ws = self.winfo_screenwidth()
        hs = self.winfo_screenheight()
        x = (ws/2) - (w/2)
        y = (hs/2) - (h/2)
        self.geometry('+%d+%d' % (x, y))
        
        self.pk = pk2c()
        self.geometry('1100x500')
        self.frame1 = Frame(self)
        self.frame2 = Frame(self, padx=10)
        self.frame3 = Frame(self)
        self.frame4 = Frame(self)
        self.frame5 = Frame(self)
        self.frame6 = Frame(self)
        self.frame7 = Frame(self)
        self.frame1.grid(column=0, row=2)
        self.frame2.grid(column=0, row=1)
        self.frame3.grid(column=2, row=2)
        self.frame4.grid(column=2, row=1)
        self.frame5.grid(column=1, row=1)
        self.frame6.grid(column=0, row=0)
        self.frame7.grid(column=2, row=0)
        
        if os.path.basename(os.path.normpath(pathlib.Path().absolute())) != 'Data':
            os.chdir(pathlib.Path('Data').absolute())

        #Get lists of files
        self.filenames = glob.glob('*.{}'.format('csv'))
        # Combobox creation 
        self.chosen_file = tk.StringVar() 
        self.file_combobox = ttk.Combobox(self.frame5, width = 20, textvariable = self.chosen_file) 

        # Adding combobox drop down list 
        self.file_combobox['values'] = self.filenames
        self.file_combobox.grid(column = 0, row = 1) 
        self.file_combobox.current(0)
        
        #Creating blank canvas to plot input function on
        self.fig1 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        self.canvas1 = FigureCanvasTkAgg(self.fig1, master=self.frame2)
        self.canvas1.get_tk_widget().grid(row=0, column=0)
        self.canvas1.draw()
        toolbar = NavigationToolbar2Tk(self.canvas1, self.frame6)
        toolbar.update()
        
        #Creating blank canvas to plot output function on
        self.fig2 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        self.canvas2 = FigureCanvasTkAgg(self.fig2, master=self.frame4)
        self.canvas2.get_tk_widget().grid(row=0, column=0)
        self.canvas2.draw()
        toolbar = NavigationToolbar2Tk(self.canvas2, self.frame7)
        toolbar.update()
        
        
        #Creating buttons to load data and perform optimization
        self.load_btn_text = tk.StringVar()
        self.load_btn = Button(master=self.frame5, textvariable=self.load_btn_text, command=self.load_data)
        self.load_btn_text.set("Load")
        self.load_btn.grid(column=0, row=2)

        #Creating button to go back to main page
        self.back_btn = Button(master=self.frame5, text="Go Back", command=self.go_back)
        self.back_btn.grid(column=0, row=4)

# #-------------------------------------------------------------
# #-------------------------------------------------------------
# #-------------------------------------------------------------

    def load_data(self):
        
        file = self.file_combobox.get()
        self.legend.append(file)
        self.pk.time = []
        self.pk.aorta = []
        self.pk.myo = []
        self.pk.Get_data(file)

        self.fig1.add_subplot(111).plot(self.pk.time, self.pk.aorta)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.fig1.canvas.draw()

        self.fig2.add_subplot(111).plot(self.pk.time, self.pk.myo)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.canvas2.draw_idle()
        
        plt.legend(self.legend)
        
    def go_back(self):
        self.destroy()

#=================================================================================================================

class pk_visualize(Toplevel):
    
    def __init__(self, parent):#, master_root):
        super().__init__(name='pk Optimizer tool')
        self.parent = parent
        
        #Making mainfraim to center the window to the screen
        mainframe = ttk.Frame(self)
        mainframe.columnconfigure(0, weight=1)
        mainframe.rowconfigure(0, weight=1)
        
        #Calculating the center
        w = 1280
        h = 500
        ws = self.winfo_screenwidth()
        hs = self.winfo_screenheight()
        x = (ws/2) - (w/2)
        y = (hs/2) - (h/2)
        self.geometry('+%d+%d' % (x, y))
        
        self.pk = pk2c()
        self.geometry('1280x500')
        self.frame1 = Frame(self)
        self.frame2 = Frame(self)
        self.frame3 = Frame(self)
        self.frame4 = Frame(self)
        self.frame5 = Frame(self)
        
        self.frame1.grid(column=1, row=0)
        self.frame2.grid(column=1, row=1)
        self.frame3.grid(column=2, row=1)
        self.frame4.grid(column=3, row=0)
        self.frame5.grid(column=3, row=1)
        
        if os.path.basename(os.path.normpath(pathlib.Path().absolute())) != 'Data':
            os.chdir(pathlib.Path('Data').absolute())

        #Get lists of files
        self.filenames = glob.glob('*.{}'.format('csv'))
        # Combobox creation 
        self.chosen_file = tk.StringVar() 
        self.file_combobox = ttk.Combobox(self.frame3, width = 20, textvariable = self.chosen_file) 

        # Adding combobox drop down list 
        self.file_combobox['values'] = self.filenames
        self.file_combobox.grid(column = 1, row = 0) 
        self.file_combobox.current(0)
        
        #Creating blank canvas to plot input function on
        self.fig1 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        #Creating blank canvas to plot output function on
        self.fig2 = plt.figure(figsize=(4.5, 4.5), dpi = 100)
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        
        self.canvas1 = FigureCanvasTkAgg(self.fig1, master=self.frame2)
        self.canvas1.get_tk_widget().grid(row=0, column=0)
        self.canvas1.draw()
        toolbar = NavigationToolbar2Tk(self.canvas1, self.frame1)
        toolbar.update()
        
        self.canvas2 = FigureCanvasTkAgg(self.fig2, master=self.frame5)
        self.canvas2.get_tk_widget().grid(row=0, column=0)
        self.canvas2.draw()
        toolbar = NavigationToolbar2Tk(self.canvas2, self.frame4)
        toolbar.update()
        
        #Creating a bunch of labels and input boxes for initial guesses for parameters
        self.ymax_lbl = Label(self.frame3, text="ymax: ")
        self.ymax_lbl.grid(column=0, row=1)
        self.tmax_lbl = Label(self.frame3, text="tmax: ")
        self.tmax_lbl.grid(column=0, row=2)
        self.alpha_lbl = Label(self.frame3, text="alpha: ")
        self.alpha_lbl.grid(column=0, row=3)
        self.delay_lbl = Label(self.frame3, text="delay: ")
        self.delay_lbl.grid(column=0, row=4)
        
        self.ymax_slider = Scale(self.frame3, from_=10, to=500, orient=HORIZONTAL, length=250, resolution=1)
        self.ymax_slider.set(250)
        self.ymax_slider.grid(column=1,row=1)
        self.tmax_slider = Scale(self.frame3, from_=0.0, to=20, orient=HORIZONTAL, length=250, resolution=0.5)
        self.tmax_slider.set(6)
        self.tmax_slider.grid(column=1,row=2)
        self.alpha_slider = Scale(self.frame3, from_=0, to=10, orient=HORIZONTAL, length=250, resolution=0.25)
        self.alpha_slider.set(2)
        self.alpha_slider.grid(column=1,row=3)
        self.delay_slider = Scale(self.frame3, from_=0, to=10, orient=HORIZONTAL, length=250, resolution=1)
        self.delay_slider.set(0)
        self.delay_slider.grid(column=1,row=4)
        
#         def show_values():
#             print (w1.get(), w2.get())

        self.flow_lbl = Label(self.frame3, text="flow:")
        self.flow_lbl.grid(column=0, row=6)
        self.visf_lbl = Label(self.frame3, text="visf:")
        self.visf_lbl.grid(column=0, row=7)
        self.baseline_lbl = Label(self.frame3, text="baseline: ")
        self.baseline_lbl.grid(column=0, row=8)
        
        self.flow_slider = Scale(self.frame3, from_=0, to=.1, orient=HORIZONTAL, length=250, resolution=0.0025)
        self.flow_slider.set(0.03)
        self.flow_slider.grid(column=1,row=6)
        self.visf_slider = Scale(self.frame3, from_=0, to=10, length=250, orient=HORIZONTAL, resolution=0.1)
        self.visf_slider.set(2)
        self.visf_slider.grid(column=1,row=7)
        self.baseline_slider = Scale(self.frame3, from_=20, to=80, length=250, orient=HORIZONTAL, resolution=1)
        self.baseline_slider.set(50)
        self.baseline_slider.grid(column=1,row=8)
        
        #Creating buttons to load data and perform optimization
        self.load_btn_text = tk.StringVar()
        self.load_btn = Button(master=self.frame3, textvariable=self.load_btn_text, command=self.load_data, padx=15)
        self.load_btn_text.set("Load")
        self.load_btn.grid(column=2, row=0)

        self.opt_btn_text = tk.StringVar()
        self.opt_btn = Button(master=self.frame3, textvariable=self.opt_btn_text, command=self.plot)
        self.opt_btn_text.set("Plot")
        self.opt_btn.grid(column=2, row=3)
        
        #Creating button to go back to main page
        self.back_btn = Button(master=self.frame3, text="Go Back", command=self.go_back)
        self.back_btn.grid(column=1, row=9)

# #-------------------------------------------------------------
# #-------------------------------------------------------------
# #-------------------------------------------------------------
    def plot(self):
        file = self.file_combobox.get()
        self.pk.Get_data(file)
        
        self.pk.ymax = self.ymax_slider.get()
        self.pk.tmax = self.tmax_slider.get()
        self.pk.alpha = self.alpha_slider.get()
        self.pk.delay = self.delay_slider.get()
        
        self.fig1.add_subplot(111).plot(np.arange(0, 25, 0.01), self.pk.gamma_var(np.arange(0, 25, 0.01),
                                                        self.ymax_slider.get(),
                                                        self.tmax_slider.get(),
                                                        self.alpha_slider.get(),
                                                        self.delay_slider.get()), 'k-')
        
        self.fig1.canvas.draw()
        
#         #Performs function fitting, outputs optimized parameters in opt
#         opt = self.pk.outputFuncFit([self.flow_slider.get(),self.visf_slider.get(), self.baseline_slider.get()])
#         self.flow_slider.set(opt[0])
#         self.visf_slider.set(opt[1])
#         self.baseline_slider.set(opt[2])
        
        self.pk.flow = self.flow_slider.get()
        self.pk.visf = self.visf_slider.get()
        self.pk.baseline = self.baseline_slider.get()
    
        #Plots fitted gamma_var curve
        deriv_sol = solve_ivp(self.pk.derivs, [0, 30], [0, 0], t_eval=self.pk.time)
        fit_myo = deriv_sol.y[0] + deriv_sol.y[1]
        self.fig2.add_subplot(111).plot(self.pk.time, fit_myo + self.pk.baseline, 'm-')
        self.canvas2.draw_idle()
        
    def load_data(self):
        
        file = self.file_combobox.get()
        self.pk.Get_data(file)

        self.fig1.add_subplot(111).plot(self.pk.time, self.pk.aorta, 'bo')
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.fig1.canvas.draw()

        self.fig2.add_subplot(111).plot(self.pk.time, self.pk.myo, 'ro')
        plt.ylabel("Mass of Tracer (mg)")
        plt.xlabel("Time (sec)")
        self.canvas2.draw_idle()
     
    def go_back(self):
        self.destroy()        
        
#=================================================================================================
if __name__ == "__main__":
    mainWindow = Tk()
    gui = StartPage(mainWindow)
    mainWindow.mainloop()


# In[ ]:




